def get_schema():
    return {
            "sound-mappings": {
                "description": "Root node for hippasos specific entries.",
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "name": {
                            "description": "unique name for sound event",
                            "type": "string"
                        },
                        "sound-file": {
                            "description": "uri to sound file. must be ogg or wav.",
                            "type": "string"
                        },
                        "topic-sub": {
                            "description": "react to published values on this channel",
                            "type": "string"
                        },
                        "message-value": {
                            "description": "react to this message content",
                            "type": "string"
                        },
                        "volume": {
                            "description": "0..1 - volume relative to system volume",
                            "type": "number",
                            "minimum": 0,
                            "maximum": 1
                        },
                        "active": {
                            "description": "",
                            "type": "boolean"
                        }
                    },
                    "required": ["active", "volume", "message-value", "topic-sub", "sound-file", "name"],
                    "additionalProperties": False
                },
                "additionalProperties": False
            }
           }



import hippasos.schema as schema
from pelops.abstractmicroservice import AbstractMicroservice
from hippasos.soundevent import SoundEvent
import pygame


class SoundService(AbstractMicroservice):
    """
    Hippasos plays preconfigured sound files upon reception of predefined mqtt messages. The events are handled
    independently.

    mqtt:
        mqtt-address: localhost
        mqtt-port: 1883
        mqtt-credentials: ~/credentials.yaml
        log-level: WARNING

    logger:
        log-level: DEBUG
        log-file: hippasos.log

    sound-mappings:
        - name: bell_building  # unique name for sound event
          sound-file: ../resources/church_bell.ogg  # uri to sound file. must be ogg or wav.
          topic-sub: /test/button1  # react to published values on this channel
          message-value: PRESSED  # react to this message content
          volume: 0.1  # 0..1 - volume relative to system volume
          active: True  # entry ignored if set to False
    """

    _sound_events = None  # list of registered sound events

    def __init__(self, config, mqtt_client=None, logger=None):
        """
        Constructor.

        :param config: config yaml structure
        :param mqtt_client: instance of an mqtt client (optional)
        :param logger: instance of a logger (optional)
        """

        AbstractMicroservice.__init__(self, config, "sound-mappings", mqtt_client, logger)

        self._logger.debug("SoundService.__init__ - initializing pygame")
        pygame.init()
        pygame.mixer.init()

        self._logger.info("SoundService.__init__ initializing sound events")
        self._sound_events = []
        for entry in self._config:
            if entry["active"]:
                sound_event = SoundEvent(entry, self._mqtt_client, self._logger, pygame.mixer)
                self._sound_events.append(sound_event)

    @classmethod
    def _get_schema(cls):
        return schema.get_schema()

    def _start(self):
        """
        Starts all sound events.
        """
        for sound_event in self._sound_events:
            sound_event.start()

    def _stop(self):
        """
        Stops all sound events.
        """
        for sound_event in self._sound_events:
            sound_event.stop()

    @classmethod
    def _get_description(cls):
        return "Mqtt microservice to play sounds."


def standalone():
    SoundService.standalone()


if __name__ == "__main__":
    SoundService.standalone()



from setuptools import setup, find_packages
import os
import hippasos


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


if os.path.isfile(os.path.join(os.path.dirname(__file__), 'README.md')):
    from pypandoc import convert
    readme_rst = convert(os.path.join(os.path.dirname(__file__), 'README.md'), 'rst')
    with open(os.path.join(os.path.dirname(__file__), 'README.rst'), 'w') as out:
        out.write(readme_rst + '\n')

setup(
    name='Hippasos',
    version=hippasos.version,
    packages=find_packages(),
    license='MIT license',
    long_description=read('README.rst'),
    description='Hippasos plays preconfigured sound files upon reception of predefined mqtt messages.',
    url='https://gitlab.com/pelops/lysidike/',
    author='Tobias Gawron-Deutsch',
    author_email='tobias@strix.at',
    keywords='mqtt device driver rpi raspberry pi',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "Topic :: Home Automation",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.5',
    install_requires=[
        "pelops>=0.1a9",
        "pygame",
    ],
    entry_points={
        'console_scripts': [
            'hippasos = hippasos.soundservice:standalone',
        ]
    },

)



import os
from pelops import mylogger


class SoundEvent:
    """
    Waits for the specified message on the subscribed topic. Upon reception the sound is played.
    """

    _config = None  # yaml config structure
    _logger = None  # logger

    _name = None  # name of sound event
    _message_value = None  # play sound if this value is received
    _mqtt_client = None  # mqtt_client instance

    _pygame_mixer = None  # mixer instance from pygame
    _pygame_sound = None  # pygame sound instance containing the provided sound file
    _volume = None  # 0..1 volume setting

    def __init__(self, config, mqtt_client, parent_logger, pygame_mixer):
        """
        Constructor

        :param config: yaml config structure
        :param mqtt_client: mqtt_client instance
        :param logger: logger instance from the parent. a child will be spawned
        :param pygame_mixer: playback instance
        """
        self._config = config
        self._logger = mylogger.get_child(parent_logger, self.__class__.__name__)
        self._name = self._config["name"]

        self._logger.info("SoundEvent.__init__ - initializing {}.".format(self._name))
        self._logger.debug("SoundEvent.__init__ - config: ".format(self._config))

        self._pygame_mixer = pygame_mixer
        self._logger.debug("SoundEvent.__init__ - loading wav-file: ".format(self._config["sound-file"]))
        self._pygame_sound = self._pygame_mixer.Sound(os.path.expanduser(self._config["sound-file"]))

        self._volume = float(self._config["volume"])
        if not (0 <= self._volume <= 1):
            err = "SoundEvent.__init__ - volume must be 0<=x<=1 (value in config: {}).".format(self._config["volume"])
            self._logger.error(err)
            raise ValueError(err)
        self._pygame_sound.set_volume(self._volume)

        self._mqtt_client = mqtt_client
        self._message_value = self._config["message-value"]

    def _handler(self, value):
        value = value.decode("utf-8")
        if self._message_value == value:
            self.play()
        else:
            self._logger.debug("SoundEvent._handler ({}) - received unknown value '{}'. expected value '{}'".
                               format(self._name, value, self._message_value))

    def play(self):
        """
        Starts playing sound. If the sound is already beeing played, playback will be stopped and started from
        start.
        """
        self._logger.info("SoundEvent.play ({}) - playing sound.".format(self._name))
        self._pygame_sound.stop()
        self._pygame_sound.play()

    def start(self):
        """
        Subscribes to topic-sub.
        """
        self._logger.info("SoundEvent.start ({}) - subscribing to '{}'.".
                          format(self._name, self._config["topic-sub"]))
        self._mqtt_client.subscribe(self._config["topic-sub"], self._handler)

    def stop(self):
        """
        Unsubscribes from topic-sub and stops playback.
        """
        self._logger.info("SoundEvent.stop ({}) - unsubscribing from '{}'.".
                          format(self._name, self._config["topic-sub"]))
        self._mqtt_client.unsubscribe(self._config["topic-sub"], self._handler)
        self._pygame_sound.stop()
